@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -63,6 +63,7 @@ import java.util.ServiceLoader;
 import java.util.Set;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
+import java.util.zip.ZipException;
 
 import javax.lang.model.SourceVersion;
 import javax.tools.FileObject;
@@ -123,7 +124,7 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
         REVERSE {
             @Override
             public int compare(Path f1, Path f2) {
-                return -f1.getFileName().compareTo(f2.getFileName());
+                return f2.getFileName().compareTo(f1.getFileName());
             }
         }
     }
@@ -491,7 +492,7 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
 
             java.util.List<Path> files;
             try (Stream<Path> s = Files.list(d)) {
-                files = (sortFiles == null ? s : s.sorted(sortFiles)).collect(Collectors.toList());
+                files = (sortFiles == null ? s : s.sorted(sortFiles)).toList();
             } catch (IOException ignore) {
                 return;
             }
@@ -564,7 +565,11 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
                 Map<String,String> env = Collections.singletonMap("multi-release", multiReleaseValue);
                 FileSystemProvider jarFSProvider = fsInfo.getJarFSProvider();
                 Assert.checkNonNull(jarFSProvider, "should have been caught before!");
-                this.fileSystem = jarFSProvider.newFileSystem(archivePath, env);
+                try {
+                    this.fileSystem = jarFSProvider.newFileSystem(archivePath, env);
+                } catch (ZipException ze) {
+                    throw new IOException("ZipException opening \"" + archivePath.getFileName() + "\": " + ze.getMessage(), ze);
+                }
             } else {
                 this.fileSystem = FileSystems.newFileSystem(archivePath, (ClassLoader)null);
             }
@@ -790,8 +795,8 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
             return null;
         }
 
-        if (file instanceof PathFileObject) {
-            return ((PathFileObject) file).inferBinaryName(path);
+        if (file instanceof PathFileObject pathFileObject) {
+            return pathFileObject.inferBinaryName(path);
         } else
             throw new IllegalArgumentException(file.getClass().getName());
     }
@@ -800,8 +805,8 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
     public boolean isSameFile(FileObject a, FileObject b) {
         nullCheck(a);
         nullCheck(b);
-        if (a instanceof PathFileObject && b instanceof PathFileObject)
-            return ((PathFileObject) a).isSameFile((PathFileObject) b);
+        if (a instanceof PathFileObject pathFileObjectA && b instanceof PathFileObject pathFileObjectB)
+            return pathFileObjectA.isSameFile(pathFileObjectB);
         return a.equals(b);
     }
 
@@ -908,8 +913,8 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
                 dir = getClassOutDir();
             } else {
                 String baseName = fileName.basename();
-                if (sibling != null && sibling instanceof PathFileObject) {
-                    return ((PathFileObject) sibling).getSibling(baseName);
+                if (sibling != null && sibling instanceof PathFileObject pathFileObject) {
+                    return pathFileObject.getSibling(baseName);
                 } else {
                     Path p = getPath(baseName);
                     Path real = fsInfo.getCanonicalFile(p);
@@ -943,8 +948,8 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
         Iterable<? extends File> files)
     {
         ArrayList<PathFileObject> result;
-        if (files instanceof Collection<?>)
-            result = new ArrayList<>(((Collection<?>)files).size());
+        if (files instanceof Collection<?> collection)
+            result = new ArrayList<>(collection.size());
         else
             result = new ArrayList<>();
         for (File f: files) {
@@ -957,17 +962,16 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
     }
 
     @Override @DefinedBy(Api.COMPILER)
-    public Iterable<? extends JavaFileObject> getJavaFileObjectsFromPaths(
-        Collection<? extends Path> paths)
-    {
+    public Iterable<? extends JavaFileObject> getJavaFileObjectsFromPaths(Collection<? extends Path> paths) {
         ArrayList<PathFileObject> result;
-        if (paths instanceof Collection<?>)
-            result = new ArrayList<>(((Collection<?>)paths).size());
-        else
+        if (paths != null) {
+            result = new ArrayList<>(paths.size());
+            for (Path p: paths)
+                result.add(PathFileObject.forSimplePath(this,
+                        fsInfo.getCanonicalFile(p), p));
+        } else {
             result = new ArrayList<>();
-        for (Path p: paths)
-            result.add(PathFileObject.forSimplePath(this,
-                    fsInfo.getCanonicalFile(p), p));
+        }
         return result;
     }
 
@@ -1100,13 +1104,10 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
 
         @Override
         public boolean equals(Object o) {
-          if (o == null || !(o instanceof PathAndContainer)) {
-            return false;
-          }
-          PathAndContainer that = (PathAndContainer) o;
-          return path.equals(that.path)
-              && container.equals(that.container)
-              && index == this.index;
+            return (o instanceof PathAndContainer pathAndContainer)
+                    && path.equals(pathAndContainer.path)
+                    && container.equals(pathAndContainer.container)
+                    && index == pathAndContainer.index;
         }
 
         @Override
@@ -1160,9 +1161,9 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
     @Override @DefinedBy(Api.COMPILER)
     public Location getLocationForModule(Location location, JavaFileObject fo) throws IOException {
         checkModuleOrientedOrOutputLocation(location);
-        if (!(fo instanceof PathFileObject))
+        if (!(fo instanceof PathFileObject pathFileObject))
             return null;
-        Path p = Locations.normalize(((PathFileObject) fo).path);
+        Path p = Locations.normalize(pathFileObject.path);
             // need to find p in location
         return locations.getLocationForModule(location, p);
     }
@@ -1190,8 +1191,8 @@ public class JavacFileManager extends BaseFileManager implements StandardJavaFil
 
     @Override @DefinedBy(Api.COMPILER)
     public Path asPath(FileObject file) {
-        if (file instanceof PathFileObject) {
-            return ((PathFileObject) file).path;
+        if (file instanceof PathFileObject pathFileObject) {
+            return pathFileObject.path;
         } else
             throw new IllegalArgumentException(file.getName());
     }
